//********************************************************************
//*** ResourcePool/SQL/RTSQLExecute.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLExecute.cc,v 1.11 2005/03/13 14:57:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "config.h"
#include "ResourcePool/SQL/RTSQLExecute.hh"
#include "ResourcePool/SQL/RTSQLExecute_Impl.hh"

#include <iostream>
#include <sstream>

namespace fatalmind {
namespace SQL {

template<class C, class TM, class RTSQLC>
class SQLExecuteWrapper
: public C
{
    public:
		SQLExecuteWrapper(RTSQLC& rtc, const std::string& sql)
		: C(sql)
		{
		}
};

template<class TM>
struct SQLExecuteCommandSelector {
	typedef RTSQLExecute_Impl<TM> Impl;
	typedef RTSQLExecute<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLExecute_SpecializedImpl<
			  TM
			, RTSQLExecute_Impl
			, PT
			, SQLExecuteWrapper<typename PT::command_t::SQLExecute, TM, RTSQLExecute<TM> >
			, RTSQLExecute<TM>
		> Impl;
	};
};

template<class TM>
RTSQLExecute<TM>::RTSQLExecute(const ResourcePool<ResourceType<TM> >& p, const std::string& a_sql)
: _realSQLExecute(
	RTSQLExecute_Impl<TM>::template newRealCommand<
		SQLExecuteCommandSelector<TM>
	>(
		  Command<ResourceType<TM> >::getFactory(p).getDatabaseType()
		, *this
		, a_sql
	)
)
{
}

template<class TM>
RTSQLExecute<TM>::~RTSQLExecute() {
}

template<class TM>
template<class T>
void RTSQLExecute<TM>::bind(const int pos, const T& val, bool nul) {
	_realSQLExecute->bind(pos, val, nul);
}

template<class TM>
void RTSQLExecute<TM>::execute(typename ResourceType<TM>::plainresource_t& hand) {
	_realSQLExecute->execute(hand);
}

template<class TM>
void RTSQLExecute<TM>::instantiations() {
	bind<short>(0, 0);
	bind<int>(0, 0);
	bind<long>(0, 0);
	bind<double>(0, 0);
	bind<float>(0, 0);
	bind<std::string>(0, "");
}

} // namespace SQL
} // namespace fatalmind
