//********************************************************************
//*** ResourcePool/SQL/RTSQLExecute_Impl.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLExecute_Impl.hh,v 1.7 2005/03/13 16:39:06 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLEXECUTE_IMPL_HH
#define RESOURCEPOOL_SQL_RTSQLEXECUTE_IMPL_HH

#include "config.h"

#include "ResourcePool/SQL/Factory.hh"

#ifdef HAVE_MYSQL
#include "ResourcePool/mysql/ResourcePool.hh"
#include "ResourcePool/mysql/Resource.hh"
#include <mysql/mysql.h>
#endif

#ifdef HAVE_ORACLE
#include "ResourcePool/oracle/ResourcePool.hh"
#include "ResourcePool/oracle/Resource.hh"
#endif


namespace fatalmind {
namespace SQL {

template<class TM>
class RTSQLExecute_Impl {
	public:
		RTSQLExecute_Impl() {
		};

		virtual void bind(const int pos, const short val, bool nul = false) = 0;
		virtual void bind(const int pos, const int val, bool nul = false) = 0;
		virtual void bind(const int pos, const long val, bool nul = false) = 0;
//		virtual void bind(const int pos, const long long val, bool nul = false) = 0;
		virtual void bind(const int pos, const float val, bool nul = false) = 0;
		virtual void bind(const int pos, const double val, bool nul = false) = 0;
		virtual void bind(const int pos, const std::string& val, bool nul = false) = 0;

		virtual void execute(SQLResource<TM>& hand) = 0;

		virtual RTSQLExecute_Impl<TM>* clone() const = 0;

		template<class CS>
		static typename CS::Impl* newRealCommand(
			  typename SQLFactory<TM>::databaseType_t a_type
			, typename CS::RTC& a_that
			, const std::string& a_sql
		) {
			switch(a_type) {
#ifdef HAVE_MYSQL
			case SQLFactory<TM>::MYSQL3 :
				return new typename CS::template Specialized<fatalmind::ResourceType::mysql>::Impl(a_that, a_sql);
				break;
#endif
#ifdef HAVE_ORACLE
			case SQLFactory<TM>::OCI9:
				return new typename CS::template Specialized<fatalmind::ResourceType::oracle>::Impl(a_that, a_sql);
				break;
#endif
			default:
				std::ostringstream oss;
				oss << "Assertaion failure (type=" << a_type << ")";
				throw Exception(oss.str());
				break;
		}
	}
};

template<class TM>
class RTSQLSelectRow_Impl
: public RTSQLExecute_Impl<TM>
{
	public:
		RTSQLSelectRow_Impl() {
		};

		virtual void bindout(const int pos, short& val) = 0;
		virtual void bindout(const int pos, int& val) = 0;
		virtual void bindout(const int pos, long& val) = 0;
//      virtual void bindout(const int pos, long long& val) = 0;
		virtual void bindout(const int pos, float& val) = 0;
		virtual void bindout(const int pos, double& val) = 0;
		virtual void bindout(const int pos, std::string& val) = 0;

		virtual bool isnull(const unsigned int pos) const = 0;
		virtual RTSQLSelectRow_Impl<TM>* clone() const = 0;
};


template<class TM, template<class TM2>class B, class RT, class CW, class RTC>
class RTSQLExecute_SpecializedImpl 
: public B<TM>
{
	public:
		RTSQLExecute_SpecializedImpl(RTC& rtc, const std::string& sql) 
		: sqlExecute_(rtc, sql)
		{
		}

		void bind(const int pos, const short val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}
		void bind(const int pos, const int val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}
		void bind(const int pos, const long val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}
/*
		void bind(const int pos, const long long val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}
*/
		void bind(const int pos, const float val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}
		void bind(const int pos, const double val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}
		void bind(const int pos, const std::string& val, bool nul = false) {
			sqlExecute_.bind(pos, val, nul);
		}

		virtual void execute(SQLResource<TM>& hand) {
			sqlExecute_.execute(
				dynamic_cast<
					SQLResourceSpecialized<TM, typename RT::resource_t>&
				>(hand).getRealResource().get_plain_resource()
			);
		}

		virtual B<TM>* clone() const {
			return cloneIt(*this);
		}

	private:
		B<TM>* cloneIt(const RTSQLExecute_Impl<TM>& x) const {
			return new RTSQLExecute_SpecializedImpl<TM, B, RT, CW, RTC>(*this);
		}

		B<TM>* cloneIt(const RTSQLSelectRow_Impl<TM>& x) const {
			// in case this is called, the overwritten clone should have been called
			throw new Exception("Assertation failure");
		}
		
	protected:
		CW sqlExecute_;
};

} // namespace SQL
} // namespace fatalmind

#endif
