//********************************************************************
//*** ResourcePool/SQL/RTSQLSelect.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelect.cc,v 1.7 2005/03/13 14:57:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/SQL/RTSQLSelect.hh"
#include "ResourcePool/SQL/RTSQLSelectRow_Impl.hh"

namespace fatalmind {
namespace SQL {

template<class C, class TM, class RTSQLC>
class SQLSelectWrapper
: public C
{
	public:
		SQLSelectWrapper(RTSQLC& rtc, const std::string& sql)
		: C(sql)
		, rtc_(rtc)
		{
		}

		SQLSelectWrapper<C, TM, RTSQLC>* clone() const {
			return new SQLSelectWrapper<C, TM, RTSQLC>(*this);
		}

		virtual bool process() {
			return rtc_.process();
		}
	private:
		RTSQLC& rtc_;
};

template<class TM>
struct SQLSelectCommandSelector {
    typedef RTSQLSelectRow_Impl<TM> Impl;
    typedef RTSQLSelect<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLSelectRow_SpecializedImpl<
			  TM
			, RTSQLSelectRow_Impl
			, PT
			, SQLSelectWrapper<typename PT::command_t::SQLSelect, TM, RTSQLSelect<TM> >
			, RTSQLSelect<TM>
		> Impl;
	};
};

template<class TM>
RTSQLSelect<TM>::RTSQLSelect(const ResourcePool<ResourceType<TM> >& p, const std::string& sql)
: _realSQLExecute(
    RTSQLExecute_Impl<TM>::template newRealCommand<
          SQLSelectCommandSelector<TM>
    >(
		  Command<ResourceType<TM> >::getFactory(p).getDatabaseType()
        , *this
        , sql
    )
)
{
}

template<class TM>
RTSQLSelect<TM>::RTSQLSelect(const RTSQLSelect<TM>& rho)
: _realSQLExecute(_realSQLExecute->clone())
{
}

template<class TM>
RTSQLSelect<TM>::~RTSQLSelect() {
}

template<class TM>
template<class T>
void RTSQLSelect<TM>::bind(const int pos, const T& val, bool nul) {
	_realSQLExecute->bind(pos, val, nul);
}

template<class TM>
template<class T>
void RTSQLSelect<TM>::bindout(const unsigned int pos, T& val) {
	_realSQLExecute->bindout(pos, val);
}

template<class TM>
void RTSQLSelect<TM>::execute(typename ResourceType<TM>::plainresource_t& hand) {
	_realSQLExecute->execute(hand);
}

template<class TM>
void RTSQLSelect<TM>::instantiations() {
	bind<short>(0, 0);
	bind<int>(0, 0);
	bind<long>(0, 0);
	bind<double>(0, 0);
	bind<float>(0, 0);
	bind<std::string>(0, "");

	{
	short x;
	bindout<short>(0, x);
	}
	{
	int x;
	bindout<int>(0, x);
	}
	{
	long x;
	bindout<long>(0, x);
	}
	{
	double x;
	bindout<double>(0, x);
	}
	{
	float x;
	bindout<float>(0, x);
	}

	{
	std::string x;
	bindout<std::string>(0, x);
	}
}

} // namespace SQL
} // namespace fatalmind
