//********************************************************************
//*** ResourcePool/SQL/RTSQLSelectRow.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelectRow.cc,v 1.6 2005/03/13 14:57:15 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/SQL/RTSQLSelectRow.hh"
#include "ResourcePool/SQL/RTSQLSelectRow_Impl.hh"

// todo, a little strange
#ifdef HAVE_ORACLE
#include "ResourcePool/oracle/oracleType.hh"
#endif

namespace fatalmind {
namespace SQL {

template<class C, class TM, class RTSQLC>
class SQLSelectRowWrapper
: public C
{
	public:
		SQLSelectRowWrapper(RTSQLC& rtc, const std::string& sql)
		: C(sql)
		, rtc_(rtc)
		{
		}

		SQLSelectRowWrapper<C, TM, RTSQLC>* clone() const {
			return new SQLSelectRowWrapper<C, TM, RTSQLC>(*this);
		}

	private:
		RTSQLC& rtc_;
};

template<class TM>
struct SQLSelectRowCommandSelector {
    typedef RTSQLSelectRow_Impl<TM> Impl;
    typedef RTSQLSelectRow<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLSelectRow_SpecializedImpl<
			  TM
			, RTSQLSelectRow_Impl
			, PT
			, SQLSelectRowWrapper<typename PT::command_t::SQLSelectRow, TM, RTSQLSelectRow<TM> >
			, RTSQLSelectRow<TM>
		> Impl;
	};
};

template<class TM>
RTSQLSelectRow<TM>::RTSQLSelectRow(const ResourcePool<ResourceType<TM> >& p, const std::string& sql)
: _realSQLExecute(
    RTSQLExecute_Impl<TM>::template newRealCommand<
          SQLSelectRowCommandSelector<TM>
    >(
		  Command<ResourceType<TM> >::getFactory(p).getDatabaseType()
        , *this
        , sql
    )
)
{
}

template<class TM>
RTSQLSelectRow<TM>::~RTSQLSelectRow() {
}

template<class TM>
template<class T>
void RTSQLSelectRow<TM>::bind(const int pos, const T& val, bool nul) {
	_realSQLExecute->bind(pos, val, nul);
}

template<class TM>
template<class T>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, T& val) {
	_realSQLExecute->bindout(pos, val);
}

template<class TM>
bool RTSQLSelectRow<TM>::isnull(const unsigned int pos) const {
	return _realSQLExecute->isnull(pos);
}

template<class TM>
void RTSQLSelectRow<TM>::execute(typename ResourceType<TM>::plainresource_t& hand) {
	_realSQLExecute->execute(hand);
}

template<class TM>
void RTSQLSelectRow<TM>::instantiations() {
	bind<short>(0, 0);
	bind<int>(0, 0);
	bind<long>(0, 0);
	bind<double>(0, 0);
	bind<float>(0, 0);
	bind<std::string>(0, "");

	{
	short x;
	bindout<short>(0, x);
	}
	{
	int x;
	bindout<int>(0, x);
	}
	{
	long x;
	bindout<long>(0, x);
	}
	{
	double x;
	bindout<double>(0, x);
	}
	{
	float x;
	bindout<float>(0, x);
	}

	{
	std::string x;
	bindout<std::string>(0, x);
	}
}

} // namespace SQL
} // namespace fatalmind
